#include <iostream>
#include <string>
#include <vector>
#include "Windows.h"

#include "rf62Xsdk.h"
#include "rf62Xtypes.h"

using namespace SDK::SCANNERS::RF62X;

void gb()
{
    std::cout << "\n";
    std::cout << "#########################################\n";
    std::cout << "#                                       #\n";
    std::cout << "#                Goodbye                #\n";
    std::cout << "#                                       #\n";
    std::cout << "#########################################\n\n";
    sdk_cleanup();
    Sleep(3000);
}

int main()
{

    std::cout << "#########################################\n";
    std::cout << "#                                       #\n";
    std::cout << "#         Search and Parameters         #\n";
    std::cout << "#                                       #\n";
    std::cout << "#########################################\n\n";

    // Initialize sdk library
    sdk_init();

    // Print return rf62X sdk version
    std::cout << "SDK version: " << sdk_version() <<"\n";
    std::cout << "\n";
    std::cout << "-----------------------------------------\n";

    // Create value for scanners vector's type
    std::vector<std::shared_ptr<rf627smart>> list;
    // Search for rf627smart devices over network
    list = rf627smart::search();

    // Print count of discovered rf627smart in network by Service Protocol
    std::cout << "\nWas found\t: " << list.size()<< " RF627-Smart\n";
    std::cout << "\n-----------------------------------------\n";

    while (true)
    {
        if ( list.size() != 0 ) break;
        else
        {
            while (true)
            {
                std::cout<< "Retry search? (y/n) ";
                std::string q;
                std::cin >> q;
                std::cout<<"\n";

                if ( q == "y" || q == "Y" )
                {
                    list = rf627smart::search();
                    std::cout << "\n";
                    std::cout << "Was found\t: " << list.size()<< " RF627 \n";
                    std::cout << "\n";
                    std::cout << "-----------------------------------------\n";
                    if (list.size() != 0 ) break;
                }
                else
                {
                    gb();
                    return false;
                }
            }

        }
    }

    std::string c = " ";
    std::cout << "\nGet devices information? (y/n): ";
    std::cin >> c;
    std::cout << "\n-----------------------------------------\n";
    while (true)
    {
        if (c == "y" || c == "Y") break;
        else { gb(); return 0;}
    }

    for (size_t i = 0; i < list.size(); i++)
    {
        std::shared_ptr<hello_info> info = list[i]->get_info();
        std::cout << "\n\nID scanner's list: " << i << std::endl;
        std::cout << "-----------------------------------------" << std::endl;
        std::cout << "Device information: " << std::endl;
        std::cout << "* Name \t: " << info->device_name() << std::endl;
        std::cout << "* Serial\t: " << info->serial_number() << std::endl;
        std::cout << "* IP Addr\t: " << info->ip_address() << std::endl;
        std::cout << "* MAC Addr\t: " << info->mac_address() << std::endl;
        std::cout << "\nWorking ranges: " << std::endl;
        std::cout << "* Zsmr, mm\t: " << info->z_smr() << std::endl;
        std::cout << "* Zmr , mm\t: " << info->z_mr() << std::endl;
        std::cout << "* Xsmr, mm\t: " << info->x_smr() << std::endl;
        std::cout << "* Xemr, mm\t: " << info->x_emr() << std::endl;
        std::cout << "\nVersions: " << std::endl;
        std::cout << "* Firmware\t: " << info->firmware_version() << std::endl;
        std::cout << "* Hardware\t: " << info->hardware_version() << std::endl;
        std::cout << "-----------------------------------------" << std::endl;
    }

    std::string r = " ";
    std::cout << "\nRead and change parameters? (y/n): ";
    std::cin >> r;
    std::cout << "\n-----------------------------------------\n";

    while (true)
    {
        if (r == "y" || r == "Y") break;
        else { gb(); return 0;}
    }

    // Iterate over all discovered scanners in network, connect to each of them,
    // get parameters and set new values.
    for (size_t i = 0; i < list.size(); i++)
    {
        std::shared_ptr<rf627smart> scanner = list[i];
        std::shared_ptr<hello_info> info = scanner->get_info();

        // Print information about the scanner to which the profile belongs.
        std::cout << "\n\nID scanner's list: " << i              << "\n";
        std::cout << "-----------------------------------------" << "\n";
        std::cout << "Device information: "                      << "\n";
        std::cout << "* Name  \t: "   << info->device_name()     << "\n";
        std::cout << "* Serial\t: "   << info->serial_number()   << "\n";
        std::cout << "* IP Addr\t: "  << info->ip_address()      << "\n";

        // Establish connection to the RF627 device by Service Protocol.
        bool is_connected = scanner->connect();
        if (!is_connected){
            std::cout << "Failed to connect to scanner!\n";
            continue;
        }
        // read params from RF627 device by Service Protocol.
        bool is_read = scanner->read_params();
        if (!is_read){
            std::cout << "Failed to read scanner parameters!\n";
            continue;
        }
        // Get parameter of Device Name
    auto name = scanner->get_param("user_general_deviceName");
    if (name != nullptr)
    {
        std::string str_name = name->getValue<std::string>();
        std::cout << "\nCurrent Device Name \t: " << str_name << "\n";
    }

    auto fps = scanner->get_param("user_sensor_framerate");
    if (fps != nullptr)
    {
        uint32_t value = fps->getValue<uint32_t>();
        std::cout << "Current FPS\t\t: "<< value << "\n";
        std::string f = " ";
        std::cout << "Edit FPS? (y/n) ";
        std::cin >> f;
        if (f == "y" || f == "Y") {
            uint32_t fp = 1;
            while(true)
            {
            std::cout<<"Enter new FPS (1-485)\t: ";
            std::cin >> value;
            if ( fp >= 1 && fp <= 485 ){
            scanner->set_param("user_sensor_framerate", value);
            std::cout<<"New FPS  \t\t: " << value << "\n";
            std::cout << "-------------------------------------\n";
            break;}
            else std::cout << "Incorrect value!\n";
            }
        }
    }

    // Get parameter of Exposure
    std::shared_ptr<param> exp = scanner->get_param("user_sensor_exposure1");
    if (exp != nullptr)
    {
        uint32_t ex = exp->getValue<uint32_t>();
        std::cout << "Sensor exposure \t: " << ex << "\n";
        std::string e = "";
        std::cout << "Edit sensor exposure? (y/n) ";
        std::cin >> e;
        if (e == "y" || e == "Y") {
            uint32_t ex = 1;
            while(true)
            {
                std::cout<<"Enter new sensor exposure (0-9847000)\t: ";
                std::cin >> ex;
                if ( ex >= 0 && ex <= 9847000 ){
                    scanner->set_param("user_sensor_exposure1", ex);
                    std::cout<<"New sensor exposure  \t: " << ex << "\n";
                    std::cout << "-------------------------------------\n";
                    break;}
                else std::cout << "Incorrect value!\n";
            }
        }
    }

    // Get parameter of Laser output
    std::shared_ptr<param> laser = scanner->get_param("user_laser_value");
    if (laser != nullptr)
    {
        uint32_t las = laser->getValue<uint32_t>();
        std::cout << "Laser output power \t: " << las << " %\n";
        std::string la = "";
        std::cout << "Edit laser power? (y/n) ";
        std::cin >> la;
        if (la == "y" || la == "Y") {
            uint32_t l = 1;
            while(true)
            {
                std::cout<<"Enter new laser power (0-100)\t: ";
                std::cin >> las;
                if ( las >= 0 && las <= 100 ){
                    scanner->set_param("user_laser_value", las);
                    std::cout<<"New laser output power  \t: " << las << " %\n";
                    std::cout << "-------------------------------------\n";
                    break;}
                else std::cout << "Incorrect value!\n";
            }
        }
        std::cout << "-------------------------------------"<< std::endl;
    }

    // Get parameter of Sync Source
    auto syncSource = scanner->get_param("user_sensor_syncSource");
    if (syncSource != nullptr)
    {
        uint32_t value = syncSource->getValue<uint32_t>();
        auto syncEnum = syncSource->getEnum<uint32_t>();
        std::cout << "Current Sync Source\t: " << syncEnum.findLabel(value)                  << std::endl;

        std::string s = "";
        std::cout << "Change Sync Source? (y/n) ";
        std::cin >> s;
        if (s == "y" || s == "Y") {
            uint32_t ss = 0;
            while(true)
            {
                std::cout<<"Select the sync mode\t: \n";
                std::cout<<"\tTo use SYNC INTERNAL, enter - 1\n";
                std::cout<<"\tTo use SYNC EXTERNAL, enter - 2\n";
                std::cin >> ss;
                if ( ss == 1 ){
                    scanner->set_param_by_key("user_sensor_syncSource", "SYNC_INTERNAL");
                    std::cout << "New Sync Source  \t: " << syncEnum.getLabel("SYNC_INTERNAL") << "\n";;
                    break;}
                if ( ss == 2 ){
                    scanner->set_param_by_key("user_sensor_syncSource", "SYNC_INTERNAL");
                    std::cout << "New Sync Source  \t: " << syncEnum.getLabel("SYNC_INTERNAL") << "\n";
                }
                else std::cout << "Incorrect value!\n";
            }
        }
        std::cout << "-------------------------------------\n";
    }

    // Apply changed parameters to the device
    std::string answer = "n";
    std::cout << "Apply changed params to the device? (y/n): ";
    std::cin >> answer;
    if (answer == "y" || answer == "Y")
    {
        scanner->write_params();
        // Save changes to the device's memory
        std::cout<<std::endl<<"Save changes to device's memory? (y/n): ";
        std::cin >> answer;
        if (answer == "y" || answer == "Y") scanner->save_params();
        else { gb(); return 0;}
    }}
    // Cleanup resources allocated with sdk_init()
    sdk_cleanup();
    gb();
    Sleep(3000);
}

